'*************************************
'*   zicog_demo v0.3                 *
'*   An emulation of the Zilog Z80   *
'*   (C) 2009 Michael Rychlik        *
'*************************************
'
'v0.0 -  1 Mar 2009 - Original version
'
'v0.1 -  4 Mar 2009 -
'
'v0.2 -  7 Mar 2009 -
'
'v0.3 -  7 Mar 2009 -
'
'This file is a minimal example of how to set up and run the zicog z80 emulator
'
CON

  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000

  ram_size = (1024 * 24)
  rom_start = $FF00

OBJ

  cpu      : "zicog"          'Zilog Z80 emulator in one COG
  fcounter : "FrequencyCount" 'Frequency counter to get execution speed
       
  kb       : "PC_Keyboard"    'You can also use Keyboard object for PS2-Keyboard                          
  TV       : "PC_Text"        'You can also use TV_Text or VGA_Text objects


VAR     

'Parameter list for CPU objects, do not change order.
long reg_base       'Address (in HUB)of Z80 register file
long ram_base       'Address (in HUB) of the Z80 RAM area
long ram_len        'Size of Z80 RAM area, assummed to start at zero in Z80 address space.
long rom_base       'Address (in HUB) of the Z80 ROM area
long rom_addr       'Address in Z80 RAM area of the start of ROM.
long io_base        'Address (in prop RAM) of the I/O contol interface
long cpu_control_ptr'Quick hack at single step control
long opcode_ptr     'Place where a fetched opcode lives.

word err            'Error code return value
long cpu_control

long frequency      'Frequency counter output goes here


CON
  'Z80 Program Status Word bits
  sign_bit   = %10000000
  zero_bit   = %01000000
  aux_bit    = %00010000
  parity_bit = %00000100
  carry_bit  = %00000001

DAT

'The Z80 Registers.
'Do not change the order of these.
'Registers are defined in pairs as words so as to enable them to be
'accessed individually as BYTE or in pairs as WORD.
'The ordering makes it easier to find a register specified by a 3-bit
'register field within an op code or to swap between main and
'alternate register sets. 
register_file word
bc_reg        word      0
de_reg        word      0
hl_reg        word      0
af_reg        word      0
bc_reg_alt    word      0 
de_reg_alt    word      0
hl_reg_alt    word      0
af_reg_alt    word      0
im_reg        word      0
ix_reg        word      0
iy_reg        word      0
sp_reg        word      0
pc_reg        word      0
  
'IO interface area. Do not change the order of these.
io_cmd_block  long
io_command    long      0
io_port       long      0            
io_data       long      0

'The Z80 will start executing instructions from here
ram_z80
              BYTE $C3, $00, $01    'JMP 0100            ; Jump to T8080 CPU disagnostic at 0100
              BYTE 0[$0100 - 3]
t8080         file "T8080.COM"

ram_z80_end

'ROM is assumed to be top of the Z80 memory map at FF00
rom_z80
              byte      0[256]
rom_z80_end

PUB start | single_step, key

  single_step := 0

  reg_base := @register_file
  ram_base := @ram_z80
  ram_len := ram_size
  rom_base := @rom_z80
  rom_addr := rom_start
  io_base := @io_cmd_block
  cpu_control_ptr := @cpu_control

  'Start the frequency counter
  fcounter.start(@frequency)

  'Start the terminal
  TV.start(12)
  TV.str(string("ZICOG v0.3",13))
  
  'Start the keyboard
  err := \kb.start(26, 27)
  TV.str (string("Starting keyboard ... "))
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))

  TV.str(string("RAM base = "))
  TV.hex(ram_base, 4)
  TV.out(13)

  TV.str(string("ROM base = "))
  TV.hex(rom_base, 4)
  TV.out(13)
  
  TV.str(string("Starting Z80 emulation..."))
    err := \cpu.start(@reg_base)
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))

  print_regs

  if single_step == 1
    repeat while kb.key == 1
    repeat
      cpu_control := $1
      repeat while cpu_control <> 0 and io_command == 0
      if io_command == 1
        TV.str(string("OUT: Port="))
        TV.hex(io_port, 2)
        TV.out(" ")
        TV.str(string("Data="))
        TV.hex(io_data, 2)
        TV.out(13)
        TV.out(13)
        io_command := 0
      elseif io_command == 2
        TV.str(string("IN: Port="))
        TV.hex(io_port, 2)
        TV.out(" ")
        TV.str(string("Data="))
        io_data := $69
        TV.hex(io_data, 2)
        TV.out(13)
        io_command := 0
      else
        print_regs
        repeat while kb.key == 0
  else
    repeat
      if io_command == 1
        TV.str(string("OUT: Port="))
        TV.hex(io_port, 2)
        TV.out(" ")
        TV.str(string("Data="))
        TV.hex(io_data, 2)
        TV.out(13)
        io_command := 0
      elseif io_command == 2
        TV.str(string("IN: Port="))
        TV.hex(io_port, 2)
        TV.out(" ")
        TV.str(string("Data="))
        io_data := $69
        TV.hex(io_data, 2)
        TV.out(13)
        io_command := 0
      TV.str(string("SPEED="))
      TV.dec(frequency * 2)
      TV.out(13)
      waitcnt (cnt + clkfreq)


PRI print_regs
  TV.str(string("PC="))
  TV.hex(pc_reg, 4)
  TV.out(" ")

  TV.hex(byte[ram_base + pc_reg], 2)
  TV.out(" ")

  TV.str(string("AF="))
  TV.hex(af_reg, 4)
  TV.out(" ")
      
  TV.str(string("BC="))
  TV.hex(bc_reg, 4)
  TV.out(" ")
      
  TV.str(string("DE="))
  TV.hex(de_reg, 4)
  TV.out(" ")

  TV.str(string("HL="))
  TV.hex(hl_reg, 4)
  TV.out(" ")
      
  TV.str(string("IX="))
  TV.hex(ix_reg, 4)
  TV.out(" ")

  TV.str(string("IY="))
  TV.hex(iy_reg, 4)
  TV.out(" ")

  TV.str(string("SP="))
  TV.hex(sp_reg, 4)
  TV.out(13)

    